/*! \file interface.c
 *  \brief DSP Driver.
 *         3DS Library.
 *
 *  \version 0.9
 *
 *  3DS Interface/Gui driver
 *
 *   Pituka - 3DS Port
 *  (c) Copyright 2009 David Colmenero (aka D_Skywalk)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>

#include <3ds.h>

#include "../global.h"
#include "game_common.h"
#include "dspfirm_cdc_bin.h"

/* snd */
#define SAMPLERATE 44100
#define SAMPLESPERBUF 1024
#define BYTESPERSAMPLE 4
#define BUFFERSIZE SAMPLESPERBUF * BYTESPERSAMPLE * 2
#define UPDATESPEREMULATEDFRAME 4

ndspWaveBuf waveBuf[2];

u32 *audioBuffer;
u32 * streamBuffer;

bool fillBlock;
int stream_offset;

extern t_CPC CPC;

int SoundSetup(void) {
    audioBuffer = (u32*)linearAlloc(BUFFERSIZE);
    memset(audioBuffer, 0, BUFFERSIZE);

    /* 3ds init dsp code */
    int error = ndspInit();
    if (error) {
        printf("NDSP - INIT - ERR!\n");
        return 0;
    }

    ndspSetOutputMode(NDSP_OUTPUT_STEREO);

    ndspChnReset(0);
    ndspChnWaveBufClear(0);

    ndspChnSetInterp(0, NDSP_INTERP_NONE);
    ndspChnSetRate(0, SAMPLERATE);
    ndspChnSetFormat(0, NDSP_FORMAT_STEREO_PCM16);


    float mix[12];
    memset(mix, 0, sizeof(mix));
    mix[0] = 1.0;
    mix[1] = 1.0;
    ndspChnSetMix(0, mix);

    memset(waveBuf, 0, sizeof(waveBuf));
    waveBuf[0].data_vaddr = &audioBuffer[0];
    waveBuf[0].nsamples = SAMPLESPERBUF;
    waveBuf[1].data_vaddr = &audioBuffer[SAMPLESPERBUF];
    waveBuf[1].nsamples = SAMPLESPERBUF;

    return SAMPLESPERBUF;

}

void SoundUpdate(void) {
    if (waveBuf[fillBlock].status != NDSP_WBUF_DONE)
        return;

    u32 * ptr = &streamBuffer[stream_offset];
    stream_offset += SAMPLESPERBUF / UPDATESPEREMULATEDFRAME;
    DSP_FlushDataCache(ptr, SAMPLESPERBUF / UPDATESPEREMULATEDFRAME);

    if (stream_offset >= SAMPLESPERBUF) {
        ndspChnWaveBufAdd(0, &waveBuf[fillBlock]);

        fillBlock = !fillBlock;

        streamBuffer = (u32*) waveBuf[fillBlock].data_pcm16;
        stream_offset = 0;
        game_run();
        return;
    }
}

void StopSound(int val) {
    if (!CPC.snd_enabled)
        return;

    if (val) {
        // clear channel 0
        ndspChnWaveBufClear(0);
    } else {
        memset(audioBuffer, 0, SAMPLESPERBUF * 2);
        DSP_FlushDataCache(audioBuffer, SAMPLESPERBUF * 2);

        ndspChnWaveBufAdd(0, &waveBuf[0]);
        ndspChnWaveBufAdd(0, &waveBuf[1]);

        fillBlock = false;
        streamBuffer = (u32*) waveBuf[fillBlock].data_pcm16;
        stream_offset = 0;
    }

}

void SoundClose(void) {
    StopSound(true);
    CPC.snd_enabled = 0;
    ndspExit();
    linearFree(audioBuffer);
}

