/*! \file abadia.c
 *  \brief abadia game driver.
 *
 *  \version 1.0
 *
 *  In Game driver
 *
 *   Pituka - 3DS Port
 *  (c) Copyright 2009 David Colmenero (aka D_Skywalk)
 */

#include <string.h>
#include <3ds.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

#include "../global.h"
#include "../3ds/game_common.h"
#include "interface.h"
#include "logros.h"
#include "patch.h"
#include "text.h"

bool need_save = false;

const u8 translate3DS[32] = { //rick pad cfg
    0x85, // A
    0x87, // B -- move adso
    0xff, // SELECT
    0x57, // START -- 57 drop obj
    0x44, // DRIGHT
    0x45, // DLEFT
    0xff, // DUP
    0xff, // DDOWN
    0x62, // R -- R
    0x83, // L -- Q
    0x56, // X -- N
    0x74, // Y -- S
    0xff, 0xff,
    0xff, // ZL
    0xff, // ZR
    0xff, 0xff, 0xff, 0xff,
    0xff, // TOUCH
    0xff, 0xff, 0xff,

    0xff, // CSTICK_RIGHT
    0xff, // CSTICK_LEFT
    0xff, // CSTICK_UP
    0xff, // CSTICK_DOWN

    0xff, // CPAD_RIGHT
    0xff, // CPAD_LEFT
    0xff, // CPAD_UP
    0xff, // CPAD_DOWN
};

extern u8 *ptrDay, *ptrHour;
extern int current_logros;
extern unsigned int remove_func;
extern unsigned int * border_color;
extern xmlWiiCFG WiitukaXML;
extern t_CPC CPC;

u16 * fullday, last_moment;
char ** current_text;

void game_help(void);

bool game_init(void) {
    logros_load(OPTS_FILE);
    current_text = (char **)text_en;
    return true;
}

/**
 * necesitamos un sistema por dias que valga para
 * los consejos de juego y para los logros que se
 * pueden conseguir en ese momento del dia...
 */
inline void game_run(void) {

    if(*fullday != last_moment) {
        logros_set();
        game_help();
        last_moment = *fullday;
    }

    // si no hay logros no necesitamos comprobar nada más...
    if(!current_logros)
        return;

    register int i = current_logros;
    while(i--)
        (*logros_ptr[i]->check_func)(logros_ptr[i]);

    if(remove_func) {
        logros_set();
        need_save = true;
    }
}

bool game_save(void) {
    if (need_save) {
        need_save = false;
        return logros_save(OPTS_FILE);
    }
    return false;
}

void game_help(void) {
    char * help_text = NULL;
    WiitukaXML.disableMenuQR = 0;
    switch (*ptrHour) {
        // llegada 1er dia
        case NONA:
            switch (*ptrDay) {
                case 1:
                    if(*border_color)
                        help_text = current_text[17];
                    else
                        help_text = current_text[0];
                    break;  // INTRO
                case 2: help_text = current_text[6]; break;  // USA LLAVE
                case 3: help_text = current_text[10]; break; // COGE LAMPARA
                case 4: help_text = current_text[11]; break; // OBEDECE
                case 6: help_text = current_text[15]; break; // RECUPERA PAPIRO
            }
            break;

        case COMPLETAS:
            switch (*ptrDay) {
                case 2:
                case 4:
                case 5:
                case 6: help_text = current_text[7]; break; // NO PUEDES DORMIR
                default:help_text = current_text[1]; break; // SIGUE ABAD
            }
            break;

        case NOCHE:
            switch (*ptrDay) {
                case 3: help_text = current_text[8];  break;  // SIGUE ENCAPUCHADO
                case 5: help_text = current_text[12]; break; // LLAVE ABAD
                case 6: help_text = current_text[13]; break; // BIBLIOTECA
                case 7: help_text = current_text[16];
                        WiitukaXML.disableMenuQR = 1;
                        ShowQR();
                        break; // ENCUENTRA EL ESPEJO
                default: help_text = current_text[3]; break; // PUEDES DORMIR
            }
            break;


        case TERCIA:
            switch (*ptrDay) {
                case 2: help_text = current_text[5]; break;  // LLAVE SCRIPTORIUM
                case 3: help_text = current_text[9]; break;  // NO PIERDAS PAPIRO
                case 6: help_text = current_text[14]; break; // GUANTES
            }
            break;

        case VISPERAS: help_text = current_text[2]; break; // FRENTE ADSO

        case PRIMA:
        case SEXTA:
            help_text = current_text[4]; break; // SIGUE ADSO

    }
    if(help_text) {
        ShowStr(help_text);
    } else
        ShowStr(CLEAN_TEXT);
}

int snapshot_save(char *pchFileName);
int snapshot_load(char *pchFileName);
extern void (*run_func) (void);
extern void (*input_func) (void);

#define STATE_INIT          0
#define STATE_CLOSE         1
#define STATE_RUN           2
#define STATE_NEW           3
#define STATE_RELOAD        4
#define STATE_BUTTON_LOAD   5
#define STATE_BUTTON_NEW    6
#define STATE_BUTTON_SAVE   7
#define STATE_QR            8

int menu_status = STATE_INIT;
extern Button buttons[MAX_BTN];

void push_button(int value) {
    int i = BTN_LOAD;
    do{
        if(i == value){
            buttons[i].status |= BTN_PUSHED;
        }else
            buttons[i].status &= ~BTN_PUSHED;
    } while(i--);
}

void handler_button(int value) {
    int result = 0;
    if(buttons[value].status & BTN_PUSHED) {
        buttons[value].status &= ~BTN_PUSHED;
        if(value == BTN_LOAD) {
            if((result = snapshot_load(SAVE_FILE)))
                printf("load error: %i\n", result);
            else
                menu_status = STATE_RELOAD;
        } else if (value == BTN_SAVE) {
            ShowSave();
            if((result = snapshot_save(SAVE_FILE)))
                printf("save error: %i\n", result);
            else
                menu_status = STATE_CLOSE;
        } else
            menu_status = STATE_NEW;

    } else
        push_button(value);

}

void game_options(void) {
    switch (menu_status) {
        case STATE_INIT:
            if(WiitukaXML.disableMenuQR && (!*border_color)) {
                menu_status = STATE_INIT;
                run_func = cpc_main;
                break;
            }
            ShowOptions();
            ShowButtons();
            StopSound(true);
            menu_status = STATE_RUN;
            break;
        case STATE_NEW:
            load_sna();
            if(WiitukaXML.language != CFG_LANGUAGE_ES)
                patch_data();
        case STATE_RELOAD:
            logros_init();
            fullday = (u16*) ptrDay;

            if(WiitukaXML.language != CFG_LANGUAGE_ES)
                current_text = (char **)text_en;
            else
                current_text = (char **)text_es;

        case STATE_CLOSE:
            video_set_palette();
            push_button(-1); // remove all pushed
            run_func = cpc_main;

            StopSound(false);
            ShowInfo();
            logros_draw();
            last_moment = 0; // reset help

            menu_status = STATE_INIT;
            buttons[BTN_BACK].status |= BTN_ENABLE;
            buttons[BTN_SAVE].status |= BTN_ENABLE;
            break;
        case STATE_RUN:
            hidScanInput();
            u32 kDown = hidKeysDown();
            if (!kDown)
                return;

            if(kDown & KEY_A) {
                handler_button(BTN_NEW);
                ShowButtons();
            } else if (kDown & KEY_Y) {
                handler_button(BTN_SAVE);
                ShowButtons();
            } else if (kDown & KEY_X) {
                handler_button(BTN_LOAD);
                ShowButtons();
            } else if(kDown & KEY_TOUCH) {
                Button * btn;
                touchPosition touch;
                register int i = MAX_BTN;

                hidTouchRead(&touch);
                while(i--){
                    if(!(buttons[i].status & BTN_ENABLE))
                        continue;
                    btn = &buttons[i];

                    if((touch.py < btn->y) || (touch.py > btn->y2) ||
                       (touch.px < btn->x) || (touch.px > btn->x2))
                        continue;

                    switch (i) {
                        case BTN_LOAD:
                        case BTN_NEW:
                        case BTN_SAVE:
                            handler_button(i);
                            break;

                        case BTN_LANG:
                            WiitukaXML.language ^= 1;
                            if(buttons[BTN_SAVE].status & BTN_ENABLE) {
                                buttons[BTN_BACK].status |= BTN_CLEAN;
                                buttons[BTN_BACK].status ^= BTN_ENABLE;
                            }
                            break;
                        case BTN_COLOR: WiitukaXML.scrtube ^= 1;  break;

                        case BTN_BACK: menu_status = STATE_CLOSE; break;
                    }
                }
                ShowButtons();
            }
            break;
    }

}
