/*! \file logros.h
 *  \brief Interfaces publicos de la libreria de logros.
 *         Libreria de 3DS.
 *
 *  \version 0.9
 *
 *  3DS Logros driver
 *
 *   Pituka - 3DS Port
 *  (c) Copyright 2016 David Colmenero (aka D_Skywalk)
 */

#ifndef logro_h
#define logro_h

#define NUM_NIGHT   2
#define NUM_LIGHT   5
#define NUM_RIGHT   8

#define NUM_SCOUTY  1
#define NUM_LIBRARY 4
#define NUM_OBEY    7

#define NUM_DECEASE 0
#define NUM_PAPYRE  3
#define NUM_FINALE  6

typedef enum logro_type {
     POS_NIGHT      = NUM_NIGHT,
     POS_LIGHT      = NUM_LIGHT,
     POS_RIGHT      = NUM_RIGHT,

     POS_SCOUTY     = NUM_SCOUTY,
     POS_LIBRARY    = NUM_LIBRARY,
     POS_OBEY       = NUM_OBEY,

     POS_DECEASE    = NUM_DECEASE,
     POS_PAPYRE     = NUM_PAPYRE,
     POS_FINALE     = NUM_FINALE,
} logro_type;

typedef enum msk_hour {
//                                 cvNsTpN
    MSK_FILTER_NIGHT_HOUR     = 0b00000001,
    MSK_FILTER_LIGHT_HOUR     = 0b00010100,
    MSK_FILTER_RIGHT_HOUR     = 0b00010100,
//                                 cvNsTpN
    MSK_FILTER_SCOUTY_HOUR    = 0b00010101,
    MSK_FILTER_LIBRARY_HOUR   = 0b00000001,
    MSK_FILTER_OBEY_HOUR      = 0b00011000,
//                                 cvNsTpN
    MSK_FILTER_DECEASE_HOUR   = 0b01100000,
    MSK_FILTER_PAPYRE_HOUR    = 0b00001000,
    MSK_FILTER_FINALE_HOUR    = 0b00000001
} msk_hour;

typedef enum msk_day {
//                                7654321
    MSK_FILTER_NIGHT_DAY      = 0b11111000,
    MSK_FILTER_LIGHT_DAY      = 0b11111000,
    MSK_FILTER_RIGHT_DAY      = 0b01111100,
//                                7654321
    MSK_FILTER_SCOUTY_DAY     = 0b11111100,
    MSK_FILTER_LIBRARY_DAY    = 0b11110000,
    MSK_FILTER_OBEY_DAY       = 0b00010000,
//                                7654321
    MSK_FILTER_DECEASE_DAY    = 0b00100000,
    MSK_FILTER_PAPYRE_DAY     = 0b00001000,
    MSK_FILTER_FINALE_DAY     = 0b10000000,
} msk_day;

typedef struct t_logro {
    logro_type pos;
    char * str;

    msk_hour hour;
    msk_day day;

    void (*check_func) (struct t_logro * logro);
    u16 addr;
    u8 val;
    u8 * ptr;
} t_logro;

void logros_init(void);
void logros_print(void);
void logros_draw(void);
void logros_set(void);
void logros_load(char * filepath);
bool logros_save(char * filepath);

#define MAX_LOGROS 9
#define SCRIPT_WILLIAM_ID 54

t_logro *logros_ptr[MAX_LOGROS];

#define NOCHE 0
#define PRIMA 1
#define TERCIA 2
#define SEXTA 3
#define NONA 4
#define VISPERAS 5
#define COMPLETAS 6

#endif
